@@ -67,8 +67,10 @@ IconItem::IconItem(QQuickItem *parent)
     connect(KIconLoader::global(), &KIconLoader::iconLoaderSettingsChanged,
             this, &IconItem::updateImplicitSize);
 
+#if QT_VERSION < QT_VERSION_CHECK(5, 10, 0)
     connect(this, &QQuickItem::enabledChanged,
             this, &IconItem::onEnabledChanged);
+#endif
 
     connect(this, &IconItem::implicitWidthChanged, this, &IconItem::implicitWidthChanged2);
     connect(this, &IconItem::implicitHeightChanged, this, &IconItem::implicitHeightChanged2);
@@ -139,6 +141,8 @@ void IconItem::setSource(const QVariant &source)
         return;
     }
 
+    disconnect(KIconLoader::global(), &KIconLoader::iconChanged, this, &IconItem::iconLoaderIconChanged);
+
     const bool oldValid = isValid();
 
     m_source = source;
@@ -207,6 +211,10 @@ void IconItem::setSource(const QVariant &source)
                     if (m_icon.isNull()) {
                         m_icon = QIcon::fromTheme(sourceString);
                     }
+
+                    //since QIcon is rendered by KIconLoader, watch for when its configuration changes now and reload as needed.
+                    connect(KIconLoader::global(), &KIconLoader::iconChanged, this, &IconItem::iconLoaderIconChanged);
+
                     m_svgIconName.clear();
                     delete m_svgIcon;
                     m_svgIcon = nullptr;
@@ -541,6 +549,12 @@ void IconItem::animationFinished()
     update();
 }
 
+void IconItem::iconLoaderIconChanged(int group)
+{
+    Q_UNUSED(group);
+    schedulePixmapUpdate();
+}
+
 void IconItem::schedulePixmapUpdate()
 {
     polish();
@@ -654,6 +668,10 @@ void IconItem::itemChange(ItemChange change, const ItemChangeData &value)
 {
     if (change == ItemVisibleHasChanged && value.boolValue) {
         m_blockNextAnimation = true;
+#if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
+    } else if (change == ItemEnabledHasChanged) {
+        onEnabledChanged();
+#endif
     } else if (change == ItemSceneChange && value.window) {
         schedulePixmapUpdate();
     }
