@@ -34,9 +34,7 @@
 #include <QMenu>
 #include <QPointer>
 
-#if (QT_VERSION > QT_VERSION_CHECK(5, 5, 0))
 #include <QPlatformSurfaceEvent>
-#endif
 
 #include <kwindowsystem.h>
 #include <KWindowSystem/KWindowInfo>
@@ -72,13 +70,13 @@ public:
     DialogPrivate(Dialog *dialog)
         : q(dialog),
           location(Plasma::Types::BottomEdge),
-          frameSvgItem(0),
+          frameSvgItem(nullptr),
           hasMask(false),
           type(Dialog::Normal),
           hideOnWindowDeactivate(false),
           outputOnly(false),
           visible(false),
-          componentComplete(dialog->parent() == 0),
+          componentComplete(dialog->parent() == nullptr),
           backgroundHints(Dialog::StandardBackground)
     {
         hintsCommitTimer.setSingleShot(true);
@@ -307,7 +305,7 @@ void DialogPrivate::updateVisibility(bool visible)
 #if HAVE_KWAYLAND
             //if is a wayland window that was hidden, we need
             //to set its position again as there won't be any move event to sync QWindow::position and shellsurface::position
-            if (shellSurface) {
+            if (shellSurface && type != Dialog::OnScreenDisplay) {
                 shellSurface->setPosition(q->position());
             }
 #endif
@@ -563,11 +561,7 @@ void DialogPrivate::updateInputShape()
     }
 
 #if HAVE_XCB_SHAPE
-    if (backgroundHints == Dialog::NoBackground) {
-        return;
-    }
-
-    if (QGuiApplication::platformName() == QStringLiteral("xcb")) {
+    if (KWindowSystem::isPlatformX11()) {
         xcb_connection_t *c = QX11Info::connection();
         static bool s_shapeExtensionChecked = false;
         static bool s_shapeAvailable = false;
@@ -590,7 +584,7 @@ void DialogPrivate::updateInputShape()
         if (outputOnly) {
             // set input shape, so that it doesn't accept any input events
             xcb_shape_rectangles(c, XCB_SHAPE_SO_SET, XCB_SHAPE_SK_INPUT,
-                                 XCB_CLIP_ORDERING_UNSORTED, q->winId(), 0, 0, 0, NULL);
+                                 XCB_CLIP_ORDERING_UNSORTED, q->winId(), 0, 0, 0, nullptr);
         } else {
             // delete the shape
             xcb_shape_mask(c, XCB_SHAPE_SO_INTERSECT, XCB_SHAPE_SK_INPUT,
@@ -733,18 +727,24 @@ void DialogPrivate::setupWaylandIntegration()
 
 
 Dialog::Dialog(QQuickItem *parent)
-    : QQuickWindow(parent ? parent->window() : 0),
+    : QQuickWindow(parent ? parent->window() : nullptr),
       d(new DialogPrivate(this))
 {
     setClearBeforeRendering(true);
     setColor(QColor(Qt::transparent));
     setFlags(Qt::FramelessWindowHint | Qt::Dialog);
 
-    setIcon(QIcon::fromTheme(QStringLiteral("plasma")));
-
     connect(this, &QWindow::xChanged, [=]() { d->slotWindowPositionChanged(); });
     connect(this, &QWindow::yChanged, [=]() { d->slotWindowPositionChanged(); });
 
+    // Given dialogs are skip task bar and don't have a decoration
+    // minimizing them using e.g. "minimize all" should just close them
+    connect(this, &QWindow::windowStateChanged, this, [this](Qt::WindowState newState) {
+        if (newState == Qt::WindowMinimized) {
+            setVisible(false);
+        }
+    });
+
     connect(this, SIGNAL(visibleChanged(bool)),
             this, SIGNAL(visibleChangedProxy()));
     connect(this, SIGNAL(visibleChanged(bool)),
@@ -782,12 +782,12 @@ void Dialog::setMainItem(QQuickItem *mainItem)
         d->hintsCommitTimer.stop();
 
         if (d->mainItem) {
-            disconnect(d->mainItem, 0, this, 0);
+            disconnect(d->mainItem, nullptr, this, nullptr);
             d->mainItem->setVisible(false);
         }
 
         if (d->mainItemLayout) {
-            disconnect(d->mainItemLayout, 0, this, 0);
+            disconnect(d->mainItemLayout, nullptr, this, nullptr);
         }
 
         d->mainItem = mainItem;
@@ -801,7 +801,7 @@ void Dialog::setMainItem(QQuickItem *mainItem)
             d->slotMainItemSizeChanged();
 
             //Extract the representation's Layout, if any
-            QObject *layout = 0;
+            QObject *layout = nullptr;
             setMinimumSize(QSize(0, 0));
             setMaximumSize(QSize(DIALOGSIZE_MAX, DIALOGSIZE_MAX));
 
@@ -879,16 +879,12 @@ QPoint Dialog::popupPosition(QQuickItem *item, const QSize &size)
             switch (d->location) {
             case Plasma::Types::TopEdge:
                 return QPoint(screen->availableGeometry().center().x() - size.width() / 2, screen->availableGeometry().y());
-                break;
             case Plasma::Types::LeftEdge:
                 return QPoint(screen->availableGeometry().x(), screen->availableGeometry().center().y() - size.height() / 2);
-                break;
             case Plasma::Types::RightEdge:
                 return QPoint(screen->availableGeometry().right() - size.width(), screen->availableGeometry().center().y() - size.height() / 2);
-                break;
             case Plasma::Types::BottomEdge:
                 return QPoint(screen->availableGeometry().center().x() - size.width() / 2, screen->availableGeometry().bottom() - size.height());
-                break;
             //Default center in the screen
             default:
                 return screen->geometry().center() - QPoint(size.width() / 2, size.height() / 2);
@@ -1042,6 +1038,16 @@ void Dialog::resizeEvent(QResizeEvent* re)
 {
     QQuickWindow::resizeEvent(re);
 
+    //it's a spontaneous event generated in qguiapplication.cpp QGuiApplicationPrivate::processWindowScreenChangedEvent
+    //QWindowSystemInterfacePrivate::GeometryChangeEvent gce(window, QHighDpi::fromNativePixels(window->handle()->geometry(), window), QRect());
+    //This happens before the first show event when there is more than one screen,
+    //right after the window has been created, the window is still 0x0,
+    //but the resize event gets delivered with 0x0 again and executed with all the bad side effects
+    //this seems to happen for every window when there are multiple screens, so something we have probably to watch out for in the future
+    if (re->size().isEmpty() || re->size() == re->oldSize()) {
+        return;
+    }
+
     //A dialog can be resized even if no mainItem has ever been set
     if (!d->mainItem) {
         return;
@@ -1180,14 +1186,12 @@ bool Dialog::event(QEvent *event)
             d->updateTheme();
         }
 #endif
-#if (QT_VERSION > QT_VERSION_CHECK(5, 5, 0))
     } else if (event->type() == QEvent::PlatformSurface) {
         const QPlatformSurfaceEvent *pSEvent = static_cast<QPlatformSurfaceEvent *>(event);
 
         if (pSEvent->surfaceEventType() == QPlatformSurfaceEvent::SurfaceCreated) {
             KWindowSystem::setState(winId(), NET::SkipTaskbar | NET::SkipPager);
         }
-#endif
     } else if (event->type() == QEvent::Show) {
         d->updateVisibility(true);
     } else if (event->type() == QEvent::Hide) {
@@ -1315,11 +1319,6 @@ void Dialog::componentComplete()
 {
     d->componentComplete = true;
     QQuickWindow::setVisible(d->visible);
-    if (d->visible) {
-        // FIXME TODO: We can remove this once we depend on Qt 5.6.1+.
-        // See: https://bugreports.qt.io/browse/QTBUG-26978
-        KWindowSystem::setState(winId(), NET::SkipTaskbar | NET::SkipPager);
-    }
     d->updateTheme();
 }
 
@@ -1366,12 +1365,12 @@ void Dialog::setVisible(bool visible)
         if (visible && d->mainItem) {
             d->mainItem->setVisible(true);
         }
+
+#if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
+        // Bug 381242: Qt remembers minimize state and re-applies it when showing
+        setWindowStates(windowStates() & ~Qt::WindowMinimized);
+#endif
         QQuickWindow::setVisible(visible);
-        if (visible) {
-            // FIXME TODO: We can remove this once we depend on Qt 5.6.1+.
-            // See: https://bugreports.qt.io/browse/QTBUG-26978
-            KWindowSystem::setState(winId(), NET::SkipTaskbar | NET::SkipPager);
-        }
         //signal will be emitted and proxied from the QQuickWindow code
     } else {
         emit visibleChangedProxy();
