@@ -60,8 +60,8 @@
 
 ContainmentInterface::ContainmentInterface(DeclarativeAppletScript *parent, const QVariantList &args)
     : AppletInterface(parent, args),
-      m_wallpaperInterface(0),
-      m_activityInfo(0),
+      m_wallpaperInterface(nullptr),
+      m_activityInfo(nullptr),
       m_wheelDelta(0),
       m_editMode(false)
 {
@@ -122,7 +122,7 @@ void ContainmentInterface::init()
 
                     QVariantHash toolboxProperties;
                     toolboxProperties[QStringLiteral("parent")] = QVariant::fromValue(this);
-                    QObject *toolBoxObject = qmlObject()->createObjectFromSource(pkg.fileUrl("mainscript"), 0, toolboxProperties);
+                    QObject *toolBoxObject = qmlObject()->createObjectFromSource(pkg.fileUrl("mainscript"), nullptr, toolboxProperties);
                     if (toolBoxObject && containmentGraphicObject) {
                         containmentGraphicObject->setProperty("toolBox", QVariant::fromValue(toolBoxObject));
                     }
@@ -422,6 +422,10 @@ void ContainmentInterface::setEditMode(bool edit)
         return;
     }
 
+    if (m_containment->immutability() != Plasma::Types::Mutable) {
+        return;
+    }
+
     m_editMode = edit;
     emit editModeChanged();
 }
@@ -446,6 +450,12 @@ void ContainmentInterface::processMimeData(QMimeData *mimeData, int x, int y, KI
 
     qDebug() << "Arrived mimeData" << mimeData->urls() << mimeData->formats() << "at" << x << ", " << y;
 
+    // Catch drops from a Task Manager and convert to usable URL.
+    if (!mimeData->hasUrls() && mimeData->hasFormat(QStringLiteral("text/x-orgkdeplasmataskmanager_taskurl"))) {
+        QList<QUrl> urls = {QUrl(QString::fromUtf8(mimeData->data(QStringLiteral("text/x-orgkdeplasmataskmanager_taskurl"))))};
+        mimeData->setUrls(urls);
+    }
+
     if (mimeData->hasFormat(QStringLiteral("text/x-plasmoidservicename"))) {
         QString data = QString::fromUtf8( mimeData->data(QStringLiteral("text/x-plasmoidservicename")) );
         const QStringList appletNames = data.split(QLatin1Char('\n'), QString::SkipEmptyParts);
@@ -567,7 +577,7 @@ void ContainmentInterface::processMimeData(QMimeData *mimeData, int x, int y, KI
 void ContainmentInterface::clearDataForMimeJob(KIO::Job *job)
 {
 #ifndef PLASMA_NO_KIO
-    QObject::disconnect(job, 0, this, 0);
+    QObject::disconnect(job, nullptr, this, nullptr);
     m_dropPoints.remove(job);
     QMenu *choices = m_dropMenus.take(job);
     m_dropJobs.remove(job);
@@ -666,7 +676,7 @@ void ContainmentInterface::mimeTypeRetrieved(KIO::Job *job, const QString &mimet
                     connect(installJob, &KJob::result, this, [this, packagePath, structure, posi](KJob *job) {
                         auto fail = [job](const QString &text) {
                             KNotification::event(QStringLiteral("plasmoidInstallationFailed"), i18n("Package Installation Failed"),
-                                                 text, QStringLiteral("dialog-error"), 0, KNotification::CloseOnTimeout, QStringLiteral("plasma_workspace"));
+                                                 text, QStringLiteral("dialog-error"), nullptr, KNotification::CloseOnTimeout, QStringLiteral("plasma_workspace"));
                         };
 
                         // if the applet is already installed, just add it to the containment
@@ -702,13 +712,11 @@ void ContainmentInterface::mimeTypeRetrieved(KIO::Job *job, const QString &mimet
                 dropActions << action;
             }
             foreach (const auto &info, appletList) {
-                QAction *action;
+                const QString actionText = i18nc("Add widget", "Add %1", info.name());
+                QAction *action = new QAction(actionText, nullptr);
                 if (!info.iconName().isEmpty()) {
-                    action = new QAction(QIcon::fromTheme(info.iconName()), info.name(), nullptr);
-                } else {
-                    action = new QAction(info.name(), nullptr);
+                    action->setIcon(QIcon::fromTheme(info.iconName()));
                 }
-
                 if (choices) {
                     choices->addAction(action);
                 }
@@ -721,7 +729,7 @@ void ContainmentInterface::mimeTypeRetrieved(KIO::Job *job, const QString &mimet
                 });
             }
             {
-                QAction *action = new QAction(i18n("Icon"), nullptr);
+                QAction *action = new QAction(i18nc("Add icon widget", "Add Icon"), nullptr);
                 if (choices) {
                     choices->addAction(action);
                 }
@@ -750,11 +758,10 @@ void ContainmentInterface::mimeTypeRetrieved(KIO::Job *job, const QString &mimet
                 }
 
                 foreach (const KPluginMetaData &info, wallpaperList) {
-                    QAction *action;
+                    const QString actionText = i18nc("Set wallpaper", "Set %1", info.name());
+                    QAction *action = new QAction(actionText, nullptr);
                     if (!info.iconName().isEmpty()) {
-                        action = new QAction(QIcon::fromTheme(info.iconName()), info.name(), nullptr);
-                    } else {
-                        action = new QAction(info.name(), nullptr);
+                        action->setIcon(QIcon::fromTheme(info.iconName()));
                     }
 
                     if (choices) {
@@ -781,13 +788,13 @@ void ContainmentInterface::mimeTypeRetrieved(KIO::Job *job, const QString &mimet
                 QAction *action;
                 if (choices) {
                     choices->addSection(i18n("Widgets"));
-                    action = choices->addAction(i18n("Icon"));
+                    action = choices->addAction(i18nc("Add icon widget", "Add Icon"));
                 } else {
                     QAction *sep = new QAction(i18n("Widgets"), nullptr);
                     sep->setSeparator(true);
                     dropActions << sep;
                     // we can at least create an icon as a link to the URL
-                    action = new QAction(i18n("Icon"), nullptr);
+                    action = new QAction(i18nc("Add icon widget", "Add Icon"), nullptr);
                     dropActions << action;
                 }
 
@@ -881,7 +888,7 @@ void ContainmentInterface::loadWallpaper()
     } else {
         if (m_wallpaperInterface) {
             m_wallpaperInterface->deleteLater();
-            m_wallpaperInterface = 0;
+            m_wallpaperInterface = nullptr;
         }
     }
 }
@@ -989,14 +996,14 @@ void ContainmentInterface::mousePressEvent(QMouseEvent *event)
     }
 
     //FIXME: very inefficient appletAt() implementation
-    Plasma::Applet *applet = 0;
+    Plasma::Applet *applet = nullptr;
     foreach (QObject *appletObject, m_appletInterfaces) {
         if (AppletInterface *ai = qobject_cast<AppletInterface *>(appletObject)) {
             if (ai->isVisible() && ai->contains(ai->mapFromItem(this, event->posF()))) {
                 applet = ai->applet();
                 break;
             } else {
-                ai = 0;
+                ai = nullptr;
             }
         }
     }
